import { Component, OnInit, Inject } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { ToastrService } from 'ngx-toastr';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

@Component({
  selector: 'app-create-task',
  templateUrl: './create-task.component.html',
  styleUrls: ['./create-task.component.css']
})
export class CreateTaskComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  taskForm: FormGroup;
  availableCommandGroups: any[] = [];
  selectedGroupCommands: any[] = [];
  availableIndividualCommands: any[] = [];
  apiUrl = `${this.baseUrl}/command-tasks`;
  editing: boolean = false;  // Flag to check if in edit mode

  constructor(
    private fb: FormBuilder,
    private http: HttpClient,
    private toastr: ToastrService,
    public dialogRef: MatDialogRef<CreateTaskComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any // Inject the task data if editing
  ) {
    this.taskForm = this.fb.group({
      commandGroup: [''],
      extraCommands: [[]],
      date: ['', Validators.required],
      time: ['', Validators.required],
      notes: ['']
    });
  }

  ngOnInit(): void {
    this.loadCommandGroups();
    this.loadIndividualCommands();

    // If task data is provided, load it into the form (edit mode)
    if (this.data && this.data.task) {
      this.editing = true;
      this.loadTaskData(this.data.task);
    }
  }

  loadCommandGroups(): void {
    this.http.get<any>(`${this.baseUrl}/command-groups`).subscribe(
      (data) => {
        this.availableCommandGroups = data['hydra:member'];
      },
      (error) => {
        this.toastr.error('Error al cargar los grupos de comandos');
      }
    );
  }

  loadIndividualCommands(): void {
    this.http.get<any>(`${this.baseUrl}/commands`).subscribe(
      (data) => {
        this.availableIndividualCommands = data['hydra:member'];
      },
      (error) => {
        this.toastr.error('Error al cargar los comandos individuales');
      }
    );
  }

  loadTaskData(task: any): void {
    // Populate form fields with task data when editing
    this.taskForm.patchValue({
      commandGroup: task.commandGroup ? task.commandGroup['@id'] : '',
      extraCommands: task.commands ? task.commands.map((cmd: any) => cmd['@id']) : [],
      date: task.dateTime ? task.dateTime.split('T')[0] : '',
      time: task.dateTime ? task.dateTime.split('T')[1].slice(0, 5) : '',
      notes: task.notes || ''
    });

    // If a command group is selected, load its commands
    if (task.commandGroup) {
      this.selectedGroupCommands = task.commandGroup.commands;
    }
  }

  onCommandGroupChange(): void {
    const selectedGroupId = this.taskForm.get('commandGroup')?.value;
    this.http.get<any>(`${this.baseUrl}/command-groups/${selectedGroupId}`).subscribe(
      (data) => {
        this.selectedGroupCommands = data.commands;
      },
      (error) => {
        this.toastr.error('Error al cargar los comandos del grupo seleccionado');
      }
    );
  }

  saveTask(): void {
    if (this.taskForm.invalid) {
      this.toastr.error('Por favor, rellene todos los campos obligatorios');
      return;
    }

    const formData = this.taskForm.value;
    const dateTime = this.combineDateAndTime(formData.date, formData.time);

    const selectedCommands = formData.extraCommands.length > 0
      ? formData.extraCommands.map((id: any) => `/commands/${id}`)
      : [""];

    const payload = {
      commandGroups: [formData.commandGroup ? `/command-groups/${formData.commandGroup}` : [""]],
      commands: selectedCommands,
      dateTime: dateTime,
      notes: formData.notes || ''
    };

    if (this.editing) {
      // Perform a PATCH request if editing an existing task
      const taskId = this.data.task.uuid;
      this.http.patch<any>(`${this.apiUrl}/${taskId}`, payload).subscribe({
        next: () => {
          this.toastr.success('Tarea actualizada con éxito');
          this.dialogRef.close(true);
        },
        error: () => {
          this.toastr.error('Error al actualizar la tarea');
        }
      });
    } else {
      // Perform a POST request if creating a new task
      this.http.post<any>(this.apiUrl, payload).subscribe({
        next: () => {
          this.toastr.success('Tarea creada con éxito');
          this.dialogRef.close(true);
        },
        error: () => {
          this.toastr.error('Error al crear la tarea');
        }
      });
    }
  }

  combineDateAndTime(date: string, time: string): string {
    const dateObj = new Date(date);
    const [hours, minutes] = time.split(':').map(Number);

    dateObj.setHours(hours, minutes, 0);
    return dateObj.toISOString();
  }

  close(): void {
    this.dialogRef.close();
  }
}
